#!/usr/bin/python
# -*- encoding: utf-8 -*-

# cteni a zapis casu do RTC obvodu MCP7940 


import smbus
import datetime
import os
import sys

adr=0x6f    # adresa obvodu


bus = smbus.SMBus(1)  # novejsi varianta RasPi (512MB)
#bus = smbus.SMBus(0)  # starsi varianta RasPi (256MB)

try:
  bus.write_byte_data(adr,0x07,0b01000000)  # Control registr nastavit na generovani 32768Hz na MFP vystupu 
except: 
  print "CHYBA - RTC obvod nekomunikuje"
  exit(0)


#=============================================
# podprogram pro zjisteni casu v obvodu RTC a jeho zobrazeni, nebo nastaveni casu v RasPi podle RTC
def rtc_info(nastav_cas=False):

  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)
  print "Systemovy cas v RasPi :" , datcas , "UTC"

  nazevdne = [ '???????', 'pondeli' , 'utery  ' , 'streda ' , 'ctvrtek' , 'patek  ' , 'sobota ' , 'nedele ' ]


  rtc =  bus.read_i2c_block_data(adr,0x00)

  stbit = (rtc[0x00] & 0b10000000) >> 7       # indikator startu oscilatoru ( "1" = bezi ; "0" = stop)

  sek = ((rtc[0x00] & 0b01110000)>>4) *10 + (rtc[0x00] & 0b00001111)   # sekundy
  min = ((rtc[0x01] & 0b01110000)>>4) *10 + (rtc[0x01] & 0b00001111)   # minuty
  hod = ((rtc[0x02] & 0b00110000)>>4) *10 + (rtc[0x02] & 0b00001111)   # hodiny

  dvt = ((rtc[0x03] & 0b00000111))                                     # den v tydnu : 1=Po  2=Ut ..... 7=Ne
  oscrunbit = ((rtc[0x03] & 0b00100000)>> 5)                           # informace o behu oscilatoru ("1"=bezi, "0"=stoji)

  den = ((rtc[0x04] & 0b00110000)>>4) *10 + (rtc[0x04] & 0b00001111)   # dny v mesici

  mes = ((rtc[0x05] & 0b00010000)>>4) *10 + (rtc[0x05] & 0b00001111)   # mesic
  lpyrbit = ((rtc[0x05] & 0b00100000)>>5)                              # info o prestupnem roku ("1"=prestupny, "0"=obycejny)

  rok = ((rtc[0x06] & 0b11110000)>>4) *10 + (rtc[0x06] & 0b00001111)   # roky 00 az 99
  rok = 2000 + rok                                                     # k rokum v RTC se pripocitava jeste stoleti 

  kalibrace = rtc[0x08]                                                # kalibracni konstanta frekvence

   
  txtcas = str(hod).rjust(2,"0") + ":" + str(min).rjust(2,"0") + ":"  + str(sek).rjust(2,"0")
  txtdat = str(nazevdne[dvt]) + "  " +  str(rok) + "-" + str(mes).rjust(2,"0") + "-" + str(den).rjust(2,"0")   

  print "Cas v RTC:     " + txtdat + " " + txtcas + " UTC" 
  if (stbit == 0):
    print "POZOR!  ST bit je 0 (oscilator nebezi)."
  if (oscrunbit == 0):
    print "POZOR!  OSCRUN bit je 0 (oscilator nebezi)."
 
  if (lpyrbit == 1):
    print "Prestupny rok."
  else:
    print "Neprestupny rok."

  print "kalibracni konstanta (adr 0x08) : ", kalibrace

  
  if (nastav_cas == True):  # pokud je parametr tohoto podprogramu True, nastavi se RasPi podle RTC
    print "Cas v RasPi byl nastaven na:"
    prikaz="sudo date -u " + (str(mes).rjust(2,"0") +
                              str(den).rjust(2,"0") +
                              str(hod).rjust(2,"0") + 
                              str(min).rjust(2,"0") + 
                              str(rok) + "." + 
                              str(sek).rjust(2,"0"))
    os.system (prikaz)




#=============================================
# podprogram pro vypis vsech registru v obvodu
def rtc_dump():
  # odradkovani mezi jednotlivymi bloky kvuli lepsimu prehledu. Cislo udava, po kterem bajtu se provede ENTER
  odradkovani = [4,9,14,16,21,23,28,31,36,41,46,51,56,61,66,71,76,81,86,91,95] 
  index_odr = 0

  print "------ Obsah vsech registru v obvodu (RTC, ALARMY, REZERVOVANE, RAM) ------"
  for adrram in range(0x60):
   
    if (adrram < 0x09):                            # oblast pro ulozeni casu a konfigurce RTC obvodu
      text = "RTC"
    elif adrram == 0x09:                           # bunka, ktera je rezervovana pro cinnost obvodu - NEPOUZIVAT!
      text = "res"
    elif adrram >=0x0A and adrram < 0x10:          # oblast pro parametry prvniho alarmu
      text = "AL0"
    elif adrram == 0x10:                           # bunka, ktera je rezervovana pro cinnost obvodu - NEPOUZIVAT!
      text = "res"
    elif adrram >=0x11 and adrram < 0x17:          # oblast pro parametry druheho alarmu
      text = "AL1"
    elif adrram == 0x17:                           # bunka, ktera je rezervovana pro cinnost obvodu - NEPOUZIVAT!
      text = "res"
    elif adrram >=0x18 and adrram < 0x20:          # oblast, ktera je rezervovana pro cinnost obvodu - NEPOUZIVAT!
      text = "RES"
    elif adrram >=0x20 and adrram < 0x60:          # oblast volne pristupne RAM
      text = "RAM"

    print text + "[" + str(adrram).rjust(2,"0") + "]=" + str( bus.read_byte_data(adr,adrram)).rjust(3,"0") + "\t",

    if (adrram == odradkovani[index_odr]):   # odradkovani po jednotlivych blocich
      print ""
      index_odr = index_odr + 1






#=============================================
# podprogram pro ulozeni systemoveho casu z RasPi do RTC obvodu
def rtc_zapis():
  datcas = datetime.datetime.utcnow()   # datcas bude obsahovat aktualni datum a cas v UTC (GMT)


  # sekundy - registr 0x00
  sekhi = int(datcas.second / 10)
  seklo = datcas.second - (10 * sekhi)
  bus.write_byte_data(adr,0x00,(sekhi*16) + seklo + 0b10000000)  # ST bit do "1"
  
  # minuty - registr 0x01
  minhi = int(datcas.minute / 10)
  minlo = datcas.minute - (10 * minhi)
  bus.write_byte_data(adr,0x01,(minhi*16) + minlo)
  
  # hodiny - registr 0x02
  hodhi = int(datcas.hour / 10)
  hodlo = datcas.hour - (10 * hodhi)
  bus.write_byte_data(adr,0x02,(hodhi*16) + hodlo)

  # den v tydnu - registr 0x03
  dvt = datetime.datetime.utcnow().isoweekday()  # Po=1 Ut=2 ....Ne=7
  bus.write_byte_data(adr,0x03,dvt)
  
  # den v mesici - registr 0x04
  denhi = int(datcas.day / 10)
  denlo = datcas.day - (10 * denhi)
  bus.write_byte_data(adr,0x04,(denhi*16) + denlo)

  # mesic - registr 0x05
  meshi = int((datcas.month) / 10)
  meslo = datcas.month - (10 * meshi)
  bus.write_byte_data(adr,0x05, (meshi*16) + meslo) 

  # rok - registr 0x06
  rokhi = int(((datcas.year) - 2000) / 10)
  roklo = (datcas.year - 2000) - (10 * rokhi)
  bus.write_byte_data(adr,0x06, (rokhi*16) + roklo) 

  print "Systemovy cas v RasPi :" , datcas , "UTC"
  print "Systemovy cas byl prekopirovan do RTC obvodu."

 

#=============================================
# podprogram pro nastaveni casu v RasPi podle NTP serveru na internetu 
def ntp():   

  print "probiha restart NTP"
  prikaz="sudo /etc/init.d/ntp restart"
  os.system (prikaz)                         # spusteni prikazu v BASHi


#=============================================
# podprogram pro cteni hodnoty z pametove bunky v RTC obvodu 
def ramread(adrram): 
  if (adrram < 0x60):
    valram = bus.read_byte_data(adr,adrram)  
    print "adresa: " , adrram , " obsahuje hodnotu: " , valram
  else:
    print "mimo rozsah"
 

#=============================================
# podprogram pro zapis hodnoty do pametove bunky v RTC obvodu 
def ramwrite(adrram,valram): 
  if (adrram < 0x60 and valram <= 0xff):
  
    # test na oblasti, ktere jso rezervovane pro cinnost obvodu - NEPOUZIVAT!
    if((adrram >=0x17 and adrram < 0x20) or adrram == 0x09 or adrram == 0x10 ):
      print "Zakazana oblast - sem neni mozne ukaldat hodnoty"
      exit ()

    bus.write_byte_data(adr,adrram,valram)  
    print "na adresu: " , adrram , " byla zapsana hodnota: " , valram
  else:
    print "mimo rozsah"



#=============================================
#  ***** hlavni program *****
#=============================================



# vyhodnoceni predavaneho parametru pri spousteni programu
try:                      # odchytavani chyby, ktera by vznikla pri chybejicim parametru
  akce = sys.argv[1]      # parametr spousteneho programu do promenne "akce"
except:                   # kdyz parametr chybi, je nahrazen prazdnym retezcem
  akce=""

akce = akce.lower()       # prevod parametru na mala pismena

if (akce == "-i"):           # informace o aktualnim case v RasPi i v RTC obvodu
  rtc_info()
  
elif (akce == "-dump"):        # vypis hodnot vsech registru v obvodu
  rtc_dump()

elif (akce == "-pi2rtc"):    # cas z RasPi zapis do RTC obvodu
  rtc_zapis()

elif (akce == "-rtc2pi"):    # nastav cas na RasPi podle RTC ovbodu
  rtc_info(True)

elif (akce == "-ntp2pi"):    # serid cas v RasPi podle internetu
  ntp()

elif (akce == "-ramw"):       # zapis hodnoty do RAM
  try:                        # odchytavani chyby, ktera by vznikla pri chybejicich parametrech
    adrram = int(sys.argv[2]) # adresa pametove bunky
    valram = int(sys.argv[3]) # hodnota pro ulozeni do pametove bunky
  except:                     # kdyz parametr chybi, je nahrazen prazdnym retezcem
    print "Spatne zadane parametry"
    exit()

  ramwrite(adrram,valram)

elif (akce == "-ramr"):       # zapis hodnoty do RAM
  try:                        # odchytavani chyby, ktera by vznikla pri chybejicich parametrech
    adrram = int(sys.argv[2]) # adresa pametove bunky
  except:                     # kdyz parametr chybi, je nahrazen prazdnym retezcem
    print "Spatne zadane parametry"
    exit()

  ramread(adrram)




else:                              # pri jakemkoli jinem parametru se zobrazi napoveda
  print "Pripustne parametry:"
  print "... -i        INFO - Jen zobrazi cas v RasPi a v RTC"
  print "... -dump     Vypis hodnot vsech registru v obvodu"
  print "... -pi2rtc   Zapise cas z RasPi do RTC"
  print "... -rtc2pi   Zapise cas z RTC do RasPi"
  print "... -ntp2pi   Restartuje NTP - seridi cas v RasPi podle internetu"
  print "  "
  print "... -ramw ADR VAL   Zapise do pameti RTC obvodu na adresu ADR hodnotu VAL"
  print "... -ramr ADR       Zobrazi hodnotu pameti RTC obvodu na adrese ADR"
  print "                     maximalni mozna adresa je 95, maximalni hodnota je 255"
  print "                        zadavani cisel pouze v desitkove soustave"



